@@ -185,7 +185,7 @@ compare.synds <- function(object, data, vars = NULL,
  for (i in 1:nplots) {
    min <- (i - 1)*nperplot + 1
    max <- min(length(commonnames),(i - 1)*nperplot + nperplot)
- # tables
+   # tables
    ttables <- vector("list",(max - min + 1))
    names(ttables) <- commonnames[min:max]
    for (j in commonnames[min:max]) {
@@ -196,19 +196,19 @@ compare.synds <- function(object, data, vars = NULL,
   # plots
    per.fact <- per.fac[per.fac$Variable %in% commonnames[min:max],]
    # per.fact <- per.fact[order(match(per.fact$Variable,commonnames[min:max])),]
-   if (stat == "percents") p <- ggplot(data=per.fact, aes(x=Value,y=Percent,fill=Data))
-   else p <- ggplot(data=per.fact, aes(x=Value,y=Count,fill=Data))
-   p <- p + geom_bar(position="dodge",colour=cols[1], stat="identity") + 
-      facet_wrap(~ Variable, scales="free", ncol=ncol)  
+   if (stat == "percents") p <- ggplot(data = per.fact, aes(x = Value, y = Percent, fill = Data))
+   else p <- ggplot(data = per.fact, aes(x = Value, y = Count, fill = Data))
+   p <- p + geom_bar(position = "dodge", colour = cols[1], stat = "identity") + 
+      facet_wrap(~ Variable, scales = "free", ncol = ncol)  
    p <- p + guides(fill = guide_legend(override.aes = list(colour = NULL))) + 
-        theme(axis.text.x=element_text(angle=-30, hjust=0, vjust=1,size=rel(rel.size.x)), 
-              legend.position="top", 
+        theme(axis.text.x = element_text(angle = -30, hjust = 0, vjust = 1, size = rel(rel.size.x)), 
+              legend.position = "top", 
               legend.key = element_rect(colour = cols[1])) 
-   p <- p + theme(legend.title=element_blank())
+   p <- p + theme(legend.title = element_blank())
    if (length(msel) > 1) p <- p + scale_fill_manual(values = c(cols[1],rep(cols[2],length(msel))))
    if (length(msel) <= 1) p <- p + scale_fill_manual(values = cols)
    plots[[i]] <- p
-  }
+ }
  
  if (length(tables) == 1) {
    tables <- tables[[1]]
@@ -319,28 +319,31 @@ dfNA <- function(data, na){
 compare.fit.synds <- function(object, data, plot = "Z",
   print.coef = FALSE, return.plot = TRUE, plot.intercept = FALSE, 
   lwd = 1, lty = 1, lcol = c("#1A3C5A","#4187BF"),
-  dodge.height = .5, point.size = 2.5, incomplete = FALSE, 
+  dodge.height = .5, point.size = 2.5, 
   population.inference = FALSE, ci.level = 0.95, ...) {   # c("#132B43", "#56B1F7")
 
- # compares and plots fits to synthetic and original data
- # first parameter must be a fit to synthetic data from glm.synds()
- # or lm.synds()
+ # Compares and plots fits to synthetic and original data
+ # First parameter must be a fit to synthetic data from glm.synds(),
+ # lm.synds(), polr.synds() or multinom.synds()
+
  value <- "Value"
- coefficient <- c("Coefficient", "Model")
+ coefficient <- c("Coefficient", "Model")  
+  
+ if (!class(object) == "fit.synds") stop("Object must have class fit.synds.\n")
+ if (!is.data.frame(data)) stop("Data must be a data frame.\n")  # theoretically can be a matrix (?)
+ if (ci.level <= 0 | ci.level > 1) stop("ci.level must be beteen 0 and 1.\n")
+
  m <- object$m
- n <- sum(object$n)
- if (is.list(object$k)) k <- sum(object$k[[1]]) else k <- sum(object$k)
+ n <- object$n
+ k <- object$k
  fitting.function <- object$fitting.function
- syn.coef    <- object$mcoefavg
+
+##?? syn.coef    <- object$mcoefavg
  
  call <- match.call()
- # if (!class(object) == "fit.synds") stop("Object must have class fit.synds\n")
- if (!is.data.frame(data)) stop("Data must be a data frame\n")  # theoretically can be a matrix (?)
- if (ci.level <= 0 | ci.level > 1) stop("ci.level must be beteen 0 and 1.\n")
- if (incomplete == TRUE & m < length(syn.coef) + 1) stop("\nYou have selected incompletely synthesised data with m less than number of coefficients.\nLack-of-fit test cannot be calculated unless m > ", length(syn.coef),".\n", call. = FALSE)
 
  # get fit to real data
- if (fitting.function %in% c("multinom")) {
+ if (fitting.function %in% c("multinom", "polr")) {
    real.fit.0 <- do.call(object$fitting.function,
                          args = list(formula = formula(object),
                          Hess = TRUE, data = call$data))
@@ -349,7 +352,7 @@ compare.fit.synds <- function(object, data, plot = "Z",
    real.fit <- summary(do.call(object$fitting.function,
                        args = list(formula = formula(object), 
                        data = call$data)))
- } else {
+ } else { ##  for glm
    real.fit <- summary(do.call(object$fitting.function,
                        args = list(formula = formula(object),
                        family = object$call$family, 
@@ -359,24 +362,65 @@ compare.fit.synds <- function(object, data, plot = "Z",
  if (object$fitting.function == "multinom") { 
    real.varcov <- vcov(real.fit.0) 
    dd <- dimnames(t(real.fit$coefficients))
-   real.coef <- as.vector(t(real.fit$coefficients))
-   real.se   <- as.vector(t(real.fit$standard.errors))
-   names(real.coef) <- names(real.se) <- 
-     paste(rep(dd[[2]], each = length(dd[[1]])), 
-     rep(dd[[1]], length(dd[[2]])), sep = ":")
-   real.fit$coefficients <- cbind(real.coef, real.se, real.coef / real.se) ##GR 12/17 clumsy to make match glm and lm 
- } else {
-   real.varcov <- real.fit$cov.unscaled   
-   if (object$fitting.function == "lm") real.varcov <- real.varcov * real.fit$sigma^2  #GR 12/17     
-   real.coef   <- real.fit$coefficients[,1] 
-   real.se     <- real.fit$coefficients[,2]  
+   real.fit$coefficients <- cbind(as.vector(t(real.fit$coefficients)), 
+                                  as.vector(t(real.fit$standard.errors)),
+                                  as.vector(t(real.fit$coefficients))/as.vector(t(real.fit$standard.errors))) 
+   dimnames(real.fit$coefficients) <- list(
+                                   paste(rep(dd[[2]], each = length(dd[[1]])), 
+                                 rep(dd[[1]], length(dd[[2]])), sep = " : "), c("Estimate","Std error","t stat"))
+ } else if (object$fitting.function == "polr") { 
+   real.varcov <- vcov(real.fit.0) 
+ } else if (object$fitting.function == "lm") {
+   real.varcov <- real.fit$cov.unscaled * real.fit$sigma^2  ##??
+ } else { # for "glm"
+   real.varcov <- real.fit$cov.scaled   
  }
 
- syn.fit  <- summary.fit.synds(object, incomplete = incomplete, 
-                               population.inference = population.inference,
-                               real.varcov = real.varcov) 
- syn.coef <- syn.fit$coefficients[,1] 
- syn.se   <- syn.fit$coefficients[,2] 
+ syn.fit  <- summary.fit.synds(object, real.varcov = real.varcov,
+                               population.inference = population.inference)
+ incomplete <- syn.fit$incomplete
+
+ # detailed results
+ res.obs <- real.fit$coefficients[,1:3]
+ colnames(res.obs) <- c("Beta","se(Beta)","Z")
+ res.syn  <- syn.fit$coefficients[,1:3] 
+ res.syn  <- res.syn[order(match(rownames(res.syn), rownames(res.obs))), ]
+ res.overlap <- compare.CI(res.syn, res.obs, ci.level = ci.level, intercept = TRUE)
+ ncoef <- nrow(res.obs) 
+
+ res.diff <-  cbind(res.syn[,1], res.obs[,1], 
+		    res.syn[,1] - res.obs[,1], 
+ 		   (res.syn[,1] - res.obs[,1]) / res.obs[,2])
+ dimnames(res.diff)[[2]] <- c("Synthetic", "Observed", "Diff", "Std. coef diff")
+#? pval <- round(2 * (1 - pnorm(abs(res.syn[,1] - res.obs[,1])/sqrt(diag(lof.varcov)))), 3)  # "p value"
+ 
+ if (incomplete == TRUE) {   
+   if (object$m < ncoef) {
+     cat("\n\nWnen some variables are not synthesised m  (= ",m,") must exceed number of",
+         "\ncoefficients (= ",ncoef,") for lack of fit test. No test can be reported.\n\n")  
+     lack.of.fit <- NULL
+     lof.pvalue  <- NULL
+   } else {
+   QB <- matrix(NA, m, length(object$mcoefavg))
+     for (i in 1:m) {
+       QB[i,] <- object$mcoef[i,] - object$mcoefavg
+     }
+     lof.varcov <- t(QB) %*% QB/(m - 1)/m
+     lack.of.fit <- t(res.diff[,3]) %*% solve(lof.varcov) %*% res.diff[,3]
+     lack.of.fit <- lack.of.fit * (object$m - ncoef)/ncoef/(object$m - 1) # Hotellings T square
+     lof.pvalue  <- 1 - pf(lack.of.fit, ncoef, object$m - ncoef)          
+   }
+ } else {
+   lof.varcov <- real.varcov*n/k/m
+   lack.of.fit <- t(res.diff[,3]) %*% solve(lof.varcov) %*% res.diff[,3]  #! multiply by m for combined test
+   lof.pvalue  <- 1 - pchisq(lack.of.fit, ncoef)
+ }
+
+##?? if (object$proper == TRUE) lof.varcov <- real.varcov * (1 + n/k)/m
+
+# Calculate summary measures
+ mean.ci.overlap   <-  mean(res.overlap[,1])
+ mean.abs.std.diff <-  mean(abs(res.diff[,4]))
  
  if (return.plot == TRUE) {
    yvar <- as.character(formula(object)[[2]])
@@ -386,7 +430,7 @@ compare.fit.synds <- function(object, data, plot = "Z",
      
      if (population.inference == FALSE) {  ## get interval from real var
        BsynCI <- BetaCI
-       for (i in c(1,3,4)) BsynCI[,i] <- BsynCI[,i] + (syn.coef - real.coef)/real.se    #!GR ci by shifting
+       for (i in c(1,3,4)) BsynCI[,i] <- BsynCI[,i] + (res.syn[,1] - res.obs[,1])/res.obs[,2]
        BsynCI[,5] <- "synthetic"
      } else {
        BsynCI <- dfCI(syn.fit, Z = TRUE, name.Z = "Z.syn", 
@@ -396,11 +440,11 @@ compare.fit.synds <- function(object, data, plot = "Z",
      title = paste0("Z values for fit to ",yvar)
    
    } else {
-     BetaCI <- dfCI(real.fit, Z = FALSE, ci.level = ci.level)  #!GR 9/17  next line name.Z changed
+     BetaCI <- dfCI(real.fit, Z = FALSE, ci.level = ci.level)  
      
      if (population.inference == FALSE) {  ## get interval from real var
        BsynCI <- BetaCI
-       for (i in c(1,3,4)) BsynCI[,i] <- BsynCI[,i] + (syn.coef - real.coef)    #!GR ci by shifting
+       for (i in c(1,3,4)) BsynCI[,i] <- BsynCI[,i] + (res.syn[,1] - res.obs[,1])
        BsynCI[,5] <- "synthetic"
      } else {
        BsynCI <- dfCI(syn.fit, Z = FALSE, name.Z = "syn.coef", 
@@ -427,54 +471,15 @@ compare.fit.synds <- function(object, data, plot = "Z",
    p <- ggplot(data = modelCI, aes_string(x = "Coefficient", y = "Value"))
    p <- p + geom_hline(yintercept = 0, colour = "grey", linetype = 2, lwd = 1)
    p <- p + CI.geom + point.geom + labs(title = title, y = xlab)
-   p <- p + scale_shape_manual(values = c(16:17), breaks = c("synthetic","observed")) +
-            scale_colour_manual(values = lcol, breaks = c("synthetic","observed"))
+   p <- p + scale_shape_manual(values = c(17:16), breaks = c("synthetic","observed")) +
+            scale_colour_manual(values = lcol[2:1], breaks = c("synthetic", "observed"))
    p <- p + coord_flip()
-   #p <- p + theme_bw()
+   # p <- p + theme_bw()
    # scale_colour_manual(values = rev(brewer.pal(3,"Blues")))
    # scale_colour_grey(start = 0, end = .6)
    p
  } else p <- NULL
 
- # detailed results
- res.obs <- real.fit$coefficients[,-4]
- colnames(res.obs) <- c("Beta","se(Beta)","Z")
- # res.syn  <- syn.fit$coefficients
- res.syn  <- syn.fit$coefficients[,-4] 
- res.syn  <- res.syn[order(match(rownames(res.syn), rownames(res.obs))), ]
- res.overlap <- compare.CI(res.syn, res.obs, ci.level = ci.level, intercept = TRUE)
- ncoef <- nrow(res.obs) 
- 
- diff <- syn.coef - real.coef
- if (incomplete == TRUE) {   ####  var cov for incomplete
-   QB <- matrix(NA, m, length(object$mcoefavg))
-     for (i in 1:m) {
-       QB[i,] <- object$mcoef[i,] - object$mcoefavg
-     }
-     lof.varcov <- t(QB) %*% QB/(m - 1)/m
- }
- else if (object$proper == TRUE) lof.varcov <- real.varcov * (1 + n/k)/m
- else lof.varcov <- real.varcov*n/k/m
-
- lack.of.fit <- t(diff) %*% solve(lof.varcov) %*% diff  #!GR multiply by m for combined test
- if (incomplete == FALSE) lof.pvalue  <- 1 - pchisq(lack.of.fit,ncoef)
- else {
-   lack.of.fit <- lack.of.fit * (object$m - ncoef)/ncoef/(object$m - 1) #!GR18/10/17 Hotellings T square
-   lof.pvalue  <- 1 - pf(lack.of.fit, ncoef, object$m - ncoef)      #!GR18/10/17 changed degrees of freedom for denom
- }
- 
- # res.diff <- compute.diff(res.syn, res.obs, intercept = TRUE, m = m, n = n, k = k)
- # res.diff <- compute.diff(syn.coef, res.obs, intercept = TRUE,m=object$m,k=k,n=dim(data)[1]) #gr
- res.diff <- compute.diff(syn.coef, res.obs, intercept = TRUE, lof.varcov = lof.varcov)
- 
- # calculate summary measures
- mean.ci.overlap   <-  mean(res.overlap[,1])
- mean.abs.std.diff <-  mean(abs(res.diff[,1]))
- 
-  #mean.lof          <-  mean(lack.of.fit)
-  #mean.lof.exp      <-  mean(lack.of.fit)/ncoef
-  #std.lof           <- (mean(lack.of.fit) - ncoef)/sqrt(2*ncoef)
-
  res <- list(call = object$call, coef.obs = res.obs, coef.syn = res.syn, 
    coef.diff = res.diff, mean.abs.std.diff = mean.abs.std.diff,
    ci.overlap = res.overlap, mean.ci.overlap =  mean.ci.overlap, 
@@ -519,9 +524,10 @@ dfCI <- function(modelsummary, names.est.se = c("Estimate","Std. Error"),
 
 
 ###-----compare.CI---------------------------------------------------------
-compare.CI <- function(synthetic, observed, ci.level, intercept, ...){
-CI <- qnorm(1- (1 - ci.level)/2)
-##Initiate
+compare.CI <- function(synthetic, observed, ci.level, intercept, ...)
+{
+ CI <- qnorm(1- (1 - ci.level)/2)
+ ##Initiate
  if(nrow(observed) > nrow(synthetic)){
    numVar <- nrow(synthetic); rNames <- rownames(synthetic)
  } else{
@@ -533,50 +539,26 @@ CI <- qnorm(1- (1 - ci.level)/2)
 ##Calculate CIoverlap
  for(i in 1:numVar) {
 
-##Store CIs
- syn.upper <- synthetic[i, 1] + (CI * synthetic[i, 2])
- syn.lower <- synthetic[i, 1] - (CI * synthetic[i, 2])
- obs.upper <- observed[i, 1] + (CI * observed[i, 2])
- obs.lower <- observed[i, 1] - (CI * observed[i, 2])
+   ##Store CIs
+   syn.upper <- synthetic[i, 1] + (CI * synthetic[i, 2])
+   syn.lower <- synthetic[i, 1] - (CI * synthetic[i, 2])
+   obs.upper <- observed[i, 1]  + (CI * observed[i, 2])
+   obs.lower <- observed[i, 1]  - (CI * observed[i, 2])
 	  
-## CI overlap
- overlap.lower <- max(obs.lower, syn.lower)       
- overlap.upper <- min(obs.upper, syn.upper)       
-#   if(overlap.lower >= overlap.upper){            #! BN-06/01/17 to comment out ??
-#	   CIoverlap[i, 1] <- 0                         #! BN-06/01/17 to comment out ??
-#	 } else {                                       #! BN-06/01/17 to comment out ??
-	   CIoverlap[i, 1] <- 0.5 * 
-       (((overlap.upper - overlap.lower) / (obs.upper - obs.lower)) + 
-		    ((overlap.upper - overlap.lower) / (syn.upper - syn.lower)))
-#	 }                                              #! BN-06/01/17 to comment out ??
-##Coef coverage
-#coverage[i, 1] = (observed[i, 1] <= syn.upper & observed[i, 1] >= syn.lower)
-##Z coverage
-#coverage[i, 2] = (observed[i, 3] <= synthetic[i, 4] + (1.96 * synthetic[i, 5]) & 
-#	 observed[i, 3] >= synthetic[i, 4] - (1.96 * synthetic[i, 5]))
+   ## CI overlap
+   overlap.lower <- max(obs.lower, syn.lower)       
+   overlap.upper <- min(obs.upper, syn.upper)       
+
+   CIoverlap[i, 1] <- 0.5 * 
+     (((overlap.upper - overlap.lower) / (obs.upper - obs.lower)) + 
+	    ((overlap.upper - overlap.lower) / (syn.upper - syn.lower)))
  }
 
-  if(intercept == FALSE) {
-    CIoverlap = CIoverlap[-1, , drop = FALSE]
-  }
+ if(intercept == FALSE) {
+   CIoverlap = CIoverlap[-1, , drop = FALSE]
+ }
   
-	return(as.data.frame(CIoverlap))
+ return(as.data.frame(CIoverlap))
 }
 
-
-###-----compute.diff-------------------------------------------------------
-compute.diff <- function(synthetic, observed, intercept, lof.varcov, ...){  
-  
-  # tempStdDiff <- cbind((synthetic[,"B.syn"] - observed[,"B"]) / synthetic[,2]) 
-  tempStdDiff <- (synthetic - observed[,1]) / observed[,2] 
-  # pval <- round(2 * (1 - pnorm(sqrt(m*k/n)*abs(tempStdDiff))), 3)
-  
-  pval <- round(2 * (1 - pnorm(abs(synthetic - observed[,1])/sqrt(diag(lof.varcov)))), 3) 
-  coefdiff <- data.frame("Std. coef diff" = tempStdDiff, 
-                         "p value" = pval, check.names = FALSE)
-
-  if(intercept == FALSE){
-    coefdiff = coefdiff[-1, ]
-  }
-  return(coefdiff)
-} 
+ 
